/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.IModel;
import cn.edu.whu.lmars.model.util.PureModelStore;

/**
 * Deletes selected figures and models in consistent.
 * 
 * @author Mingda Zhang
 * 
 */

public class GeoDeleteCommand extends GeoCommand {
	private Logger logger = Logger.getLogger(GeoDeleteCommand.class);
	private PureModelStore modelStore;
	private FigureStore figureStore;
	private List<AbstractFigure> targetFigures;

	// Store figures to be deleted
	private List<AbstractFigure> deletedFigures = new ArrayList<AbstractFigure>();
	// Store models to be deleted
	private List<IModel> deletedModels = new ArrayList<IModel>();

	/**
	 * Instantiates a GeoDeleteCommand.
	 * 
	 * @param modelStore
	 * 			model container.
	 * @param figureStore
	 * 			figure container.
	 * @param targetFigures
	 * 			selected figures to be deleted.
	 */
	public GeoDeleteCommand(PureModelStore modelStore, FigureStore figureStore,
			List<AbstractFigure> targetFigures) {
		logger.info("Create GeoDeleteCommand");
		this.modelStore = modelStore;
		this.figureStore = figureStore;
		this.targetFigures = targetFigures;
	}

	/**
	 * Executes GeoDeleteCommand.
	 */
	@Override
	public boolean execute() {
		while (targetFigures.size() > 0) {
			AbstractFigure figure = targetFigures.get(0);
			if (figure instanceof FlowFigure) {
				this.figureStore.removeFlowFigure((FlowFigure) figure);
				deletedFigures.add(figure);
				String uuid = figure.getUuid();
				ChainFlow chainFlow = modelStore.getChainFlowByUuid(uuid);
				modelStore.removeChainFlow(chainFlow);
				deletedModels.add(chainFlow);

				targetFigures.remove(figure);
				continue;
			}

			ProcessFigure processFigure = null;
			String uuid;
			if (figure instanceof ProcessFigure) {
				processFigure = (ProcessFigure) figure;
				uuid = processFigure.getUuid();
			} else {
				uuid = figure.getUuid().split("#")[0];
				processFigure = this.figureStore.getProcessFigureByUuid(uuid);
			}

			for (AbstractFigure linkfigure : processFigure.getLinkFigures()) {
				if (targetFigures.contains(linkfigure)) {
					targetFigures.remove(linkfigure);
				}
			}
			for (AbstractFigure dataFigure : processFigure.getDataFigures()) {
				if (targetFigures.contains(dataFigure)) {
					targetFigures.remove(dataFigure);
				}
			}

			figureStore.removeProcessFigure(processFigure);
			deletedFigures.add(processFigure);
			AtomProcess atomProcess = modelStore.getAtomProcessByUuid(uuid);
			modelStore.removeAtomProcess(atomProcess);
			deletedModels.add(atomProcess);
			if (targetFigures.contains(processFigure)) {
				targetFigures.remove(processFigure);
			}

			FlowFigure beginFlowfFigure = figureStore
					.getFlowFigureByBeginUuid(uuid);
			if (beginFlowfFigure != null) {
				figureStore.removeFlowFigure(beginFlowfFigure);
				deletedFigures.add(beginFlowfFigure);
				if (targetFigures.contains(beginFlowfFigure)) {
					targetFigures.remove(beginFlowfFigure);
				}

				ChainFlow chainFlow = modelStore.getChainFlowByStartUuid(uuid);
				modelStore.removeChainFlow(chainFlow);
				deletedModels.add(chainFlow);
			}

			List<FlowFigure> endsFlowFigures = figureStore
					.getFlowFiguresByEndUuid(uuid);
			if (endsFlowFigures.size() > 0) {
				for (FlowFigure flowFigure : endsFlowFigures) {
					figureStore.removeFlowFigure(flowFigure);
					deletedFigures.add(flowFigure);

					if (targetFigures.contains(flowFigure)) {
						targetFigures.remove(flowFigure);
					}
				}
			}
		}
		return true;
	}

	/**
	 * Redo GeoDeleteCommand
	 */
	@Override
	public void redo() {
		for (AbstractFigure figure : this.deletedFigures) {
			if (figure instanceof FlowFigure) {
				figureStore.removeFlowFigure((FlowFigure) figure);
			} else if (figure instanceof ProcessFigure) {
				figureStore.removeProcessFigure((ProcessFigure) figure);
			}
		}
		for (IModel model : deletedModels) {
			if (model instanceof ChainFlow) {
				modelStore.removeChainFlow((ChainFlow) model);
			} else if (model instanceof AtomProcess) {
				modelStore.removeAtomProcess((AtomProcess) model);
			}
		}
		logger.info("undo GeoDeleteCommand");
	}

	/**
	 * Undo GeoDeleteCommand.
	 */
	@Override
	public void undo() {

		for (AbstractFigure figure : this.deletedFigures) {
			if (figure instanceof FlowFigure) {
				figureStore.addFlowFigure((FlowFigure) figure);
			} else if (figure instanceof ProcessFigure) {
				figureStore.addProcessFigure((ProcessFigure) figure);
			}
		}
		for (IModel model : deletedModels) {
			if (model instanceof ChainFlow) {
				modelStore.addChainFlow((ChainFlow) model);
			} else if (model instanceof AtomProcess) {
				modelStore.addAtomProcess((AtomProcess) model);
			}
		}
		logger.info("undo GeoDeleteCommand");
	}

}
