/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.drawpanel.ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.dnd.DropTarget;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

import cn.edu.whu.lmars.commands.CommandStack;
import cn.edu.whu.lmars.drawpanel.listeners.DrawPanelMouseListener;
import cn.edu.whu.lmars.drawpanel.listeners.DrawPanelMouseMotionListener;
import cn.edu.whu.lmars.drawpanel.listeners.KeyMonitor;
import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.GeoMarkFigure;
import cn.edu.whu.lmars.figure.beans.geoenum.ButtonType;
import cn.edu.whu.lmars.figure.beans.geoenum.DragType;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.run.bind.beans.BindStore;
import cn.edu.whu.lmars.tree.listeners.dragdrop.GeoDropTargetListener;
import conf.GlobeParam;

/**
 * This is a drawing panel which stores all the information related.
 * 
 * 
 * @author Zhang Mingda
 * 
 */
@SuppressWarnings("serial")
public class JDrawPanel extends JPanel {

	/** Cache image. */
	Image offScreenImage = null;

	/** Background image. */
	private Image image;

	/** Selected mark figure. */
	private GeoMarkFigure mark;

	/** Whether to support the multiple choice. */
	private boolean mutipable = false;

	/**
	 * Button type which determines different operation when mouse events
	 * happen.
	 */
	private ButtonType btnType = ButtonType.Select;
	/** The position information of the mouse. */
	private int press_x, press_y, move_x, move_y, release_x, realse_y, drag_x,
			drag_y, last_drag_x, last_drag_y, click_x, click_y;
	/** Drag type which may leads to different actions when mouse events happen. */
	private DragType dragType = DragType.NONE;

	/** Figures are moved or not. */
	private boolean moved = false;
	/** Figures are changed or not. */
	private boolean changed = false;
	/** The path of configuration file. */
	private String modelPath = "";
	/** Command stack contains operations on this JDrawPanel. */
	private CommandStack commandStack = new CommandStack();
	/** The name of this JDrawPanel. */
	private String name;
	/** The id of this JDrawPanel. */
	private String uuid;
	/** Figure container. */
	private FigureStore figureStore;
	/** Model container. */
	private PureModelStore modelStore = null;
	/** Binding info container. */
	private BindStore bindStore = null;

	/**
	 * Instantiates a JDrawPanel with the default setting including color and
	 * background image.
	 */
	public JDrawPanel() {
		super();
		this.setBackground(Color.white);
		@SuppressWarnings("unused")
		DropTarget dropTarget = new DropTarget(this,
				new GeoDropTargetListener());
		try {
			image = ImageIO.read(getClass().getResource("/images/background.jpg"));
		} catch (IOException ex) {
			ex.printStackTrace();
		}
		initlize();
	}

	private void initlize() {
		this.addMouseListener(new DrawPanelMouseListener(this));
		this.addMouseMotionListener(new DrawPanelMouseMotionListener(this));
		this.addKeyListener(new KeyMonitor(this));
	}

	@Override
	public void paintComponents(Graphics g) {
		super.paintComponents(g);
	}

	/**
	 * When updated, redraw all graphics.
	 * 
	 * @see javax.swing.JPanel.paint(Graphics g)
	 */
	@Override
	public void paint(Graphics g) {

		// draw the background according to different situations
		if (image.getWidth(null) > this.getWidth()
				&& image.getHeight(null) > this.getHeight()) {

			// The size of the image does not exceed the size of the panel,draw
			// the panel with original size
			g.drawImage(image, 0, 0, this.getWidth(), this.getHeight(), 0, 0,
					this.getWidth(), this.getHeight(), null);
		} else {

			// enlarge image
			g.drawImage(image, 0, 0, this.getWidth(), this.getHeight(), this);
		}

		// draw the temporary figures, usually used when adding a line or resize
		// a figure
		AbstractFigure geoFigure = null;
		for (int i = 0; i < GlobeParam.TEMPORARY_FIGURES.size(); i++) {
			geoFigure = GlobeParam.TEMPORARY_FIGURES.get(i);
			geoFigure.draw(g);
		}

		// draw all the figures that the panel contains
		figureStore.draw(g);
	}

	/**
	 * Double cache to avoid flicker.
	 */
	@Override
	public void update(Graphics g) {
		if (offScreenImage == null) {
			offScreenImage = this
					.createImage(this.getWidth(), this.getHeight());
		}
		Graphics gOffScreen = offScreenImage.getGraphics();
		Color c = gOffScreen.getColor();
		gOffScreen.setColor(GlobeParam.BACKGROUD_COLOR);
		gOffScreen.fillRect(0, 0, this.getWidth(), this.getHeight());
		gOffScreen.setColor(c);
		paint(gOffScreen);
		g.drawImage(offScreenImage, 0, 0, null);

	}

	/**
	 * Returns the Mark Figure that was selected
	 * 
	 * @return Mark Figure that was selected
	 */
	public GeoMarkFigure getMark() {
		return mark;
	}

	/**
	 * Sets the selected Mark Figure.
	 * 
	 * @param mark
	 *            the selected mark figure.
	 */
	public void setMark(GeoMarkFigure mark) {
		this.mark = mark;
	}

	/**
	 * Returns whether to support the multiple choice.
	 */
	public boolean isMutipable() {
		return mutipable;
	}

	public void setMutipable(boolean mutipable) {
		this.mutipable = mutipable;
	}

	/**
	 * Returns the button type.
	 */
	public ButtonType getBtnType() {
		return btnType;
	}

	/**
	 * Returns the figure container.
	 */
	public FigureStore getFigureStore() {
		return figureStore;
	}

	/**
	 * Sets the figure container.
	 * 
	 * @param figureStore
	 */
	public void setFigureStore(FigureStore figureStore) {
		this.figureStore = figureStore;
	}

	/**
	 * Returns the model container.
	 * 
	 * @return model container.
	 */
	public PureModelStore getModelStore() {
		return modelStore;
	}

	/**
	 * Sets the model container.
	 * 
	 * @param modelStore
	 * 
	 */
	public void setModelStore(PureModelStore modelStore) {
		this.modelStore = modelStore;
	}

	/**
	 * Returns binding information container.
	 * 
	 * @return BindStore
	 */
	public BindStore getBindStore() {
		return bindStore;
	}

	/**
	 * Sets binding information container.
	 * 
	 * @param mapStore
	 */
	public void setBindStore(BindStore mapStore) {
		this.bindStore = mapStore;
	}

	/**
	 * Sets the button type.
	 * 
	 * @param btnType
	 */
	public void setBtnType(ButtonType btnType) {
		this.btnType = btnType;
	}

	/**
	 * Returns the x coordinate value of mouse pressed.
	 */
	public int getPress_x() {
		return press_x;
	}

	/**
	 * Sets the x coordinate value of mouse pressed.
	 * 
	 * @param press_x
	 */
	public void setPress_x(int press_x) {
		this.press_x = press_x;
	}

	/**
	 * Returns the y coordinate value of mouse pressed.
	 */
	public int getPress_y() {
		return press_y;
	}

	/**
	 * Sets the y coordinate value of mouse pressed.
	 * 
	 * @param press_y
	 *            the y coordinate value of mouse pressed
	 */
	public void setPress_y(int press_y) {
		this.press_y = press_y;
	}

	/**
	 * Gets the x coordinate value of mouse moved.
	 */
	public int getMove_x() {
		return move_x;
	}

	/**
	 * Sets the x coordinate value of mouse moved.
	 * 
	 * @param move_x
	 */
	public void setMove_x(int move_x) {
		this.move_x = move_x;
	}

	/**
	 * Returns the y coordinate value of mouse moved.
	 */
	public int getMove_y() {
		return move_y;
	}

	/**
	 * Sets the y coordinate value of mouse moved.
	 * 
	 * @param move_y
	 */
	public void setMove_y(int move_y) {
		this.move_y = move_y;
	}

	/**
	 * Returns the x coordinate value of mouse released.
	 */
	public int getRelease_x() {
		return release_x;
	}

	/**
	 * Sets the x coordinate value of mouse released.
	 * 
	 * @param release_x
	 *            the x coordinate value of mouse released.
	 */
	public void setRelease_x(int release_x) {
		this.release_x = release_x;
	}

	/**
	 * Returns the y coordinate value of mouse released.
	 */
	public int getRealse_y() {
		return realse_y;
	}

	/**
	 * Sets the y coordinate value of mouse released.
	 * 
	 * @param realse_y
	 *            the y coordinate value of mouse released.
	 */
	public void setRealse_y(int realse_y) {
		this.realse_y = realse_y;
	}

	/**
	 * @return the x coordinate value of mouse draged.
	 */
	public int getDrag_x() {
		return drag_x;
	}

	/**
	 * Sets the x coordinate value of mouse drags.
	 * 
	 * @param drag_x
	 *            the x coordinate value of mouse dragged.
	 */
	public void setDrag_x(int drag_x) {
		this.drag_x = drag_x;
	}

	/**
	 * Gets the y coordinate value of mouse drags.
	 */
	public int getDrag_y() {
		return drag_y;
	}

	/**
	 * Set the y coordinate value of mouse drags.
	 * 
	 * @param drag_y
	 */
	public void setDrag_y(int drag_y) {
		this.drag_y = drag_y;
	}

	/**
	 * Gets the x coordinate value of last drag.
	 */
	public int getLast_drag_x() {
		return last_drag_x;
	}

	/**
	 * Sets the x coordinate value of last drag.
	 * 
	 * @param last_drag_x
	 *            the x location of last mouse drag.
	 */
	public void setLast_drag_x(int last_drag_x) {
		this.last_drag_x = last_drag_x;
	}

	/**
	 * Gets the y coordinate value of mouse last drag.
	 */
	public int getLast_drag_y() {
		return last_drag_y;
	}

	/**
	 * Sets the y coordinate value of mouse last drag.
	 */
	public void setLast_drag_y(int last_drag_y) {
		this.last_drag_y = last_drag_y;
	}

	/**
	 * Gets the DragType.
	 * 
	 * @return DragType
	 */
	public DragType getDragType() {
		return dragType;
	}

	/**
	 * Sets the DragType.
	 */
	public void setDragType(DragType dragType) {
		this.dragType = dragType;
	}

	/**
	 * Gets whether the figures are moved.
	 */
	public boolean isMoved() {
		return moved;
	}

	/**
	 * Sets whether the figures are moved.
	 * 
	 * @param moved
	 */
	public void setMoved(boolean moved) {
		this.moved = moved;
	}

	/**
	 * Gets whether the figures are changed.
	 */
	public boolean isChanged() {
		return changed;
	}

	/**
	 * Sets whether the figures are changed.
	 * 
	 * @param changed
	 */
	public void setChanged(boolean changed) {
		this.changed = changed;
	}

	/**
	 * Gets the x coordinate value of mouse click.
	 * 
	 * @return x coordinate value of mouse click.
	 */
	public int getClick_x() {
		return click_x;
	}

	/**
	 * Sets the x coordinate value of mouse click.
	 * 
	 */
	public void setClick_x(int click_x) {
		this.click_x = click_x;
	}

	/**
	 * Gets the y coordinate value of mouse click.
	 * 
	 * @return y coordinate value of mouse click.
	 */
	public int getClick_y() {
		return click_y;
	}

	/**
	 * Sets the y coordinate value of mouse click.
	 * 
	 */
	public void setClick_y(int click_y) {
		this.click_y = click_y;
	}

	/**
	 * Gets the save path of the configuration file.
	 * 
	 * @return the save path of the configuration file.
	 */
	public String getModelPath() {
		return modelPath;
	}

	/**
	 * Sets the save path of configuration file.
	 * 
	 */
	public void setModelPath(String modelPath) {
		this.modelPath = modelPath;
	}

	/**
	 * Gets the CommandStack.
	 * 
	 * @return CommandStack
	 */
	public CommandStack getCommandStack() {
		return commandStack;
	}

	/**
	 * Gets the name of this JDrawPanel.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name of this JDrawPanel.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Gets the id of this JDrawPanel.
	 * 
	 * @return the id of this JDrawPanel.
	 */
	public String getUuid() {
		return uuid;
	}

	/**
	 * Sets the id of this JDrawPanel.
	 */
	public void setUuid(String uuid) {
		this.uuid = uuid;
	}

}
