/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.utils;

import java.io.File;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.LinkFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.util.PureModelStore;

/**
 * Gets figures from figure configuration file. File extension is '.gur'
 * 
 * @author Mingda Zhang
 * 
 */
public class OpenFigure {
	private PureModelStore modelStore;

	/**
	 * Parses the figure configuration file.
	 * 
	 * @param path
	 *            the location of configuration file.
	 * @param modelStore
	 *            the model container related to these figures.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public FigureStore open(String path, PureModelStore modelStore) {
		if (!path.endsWith(".gur")) {
			return null;
		}
		this.modelStore = modelStore;
		SAXReader reader = new SAXReader();
		FigureStore figureStore = new FigureStore();
		try {
			Document doc = reader.read(new File(path));
			Element root = doc.getRootElement();
			List<Element> processEles = root.elements("ProcessFigure");
			for (Element processEle : processEles) {
				createProcessFigure(figureStore, processEle);
			}

			List<Element> flowEles = root.elements("FlowFigure");
			for (Element flowEle : flowEles) {
				createFlowFigure(figureStore, flowEle);
			}
			return figureStore;
		} catch (DocumentException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Creates flow figure.
	 * @param figureStore
	 * 				figure container.
	 * @param element
	 * 				flow element.
	 */
	private void createFlowFigure(FigureStore figureStore, Element element) {
		String id = element.attributeValue("uuid");
		String[] uuids = id.split("#");
		ProcessFigure beginProcessFigure = figureStore
				.getProcessFigureByUuid(uuids[0]);
		ProcessFigure endProcessFigure = figureStore
				.getProcessFigureByUuid(uuids[1]);
		DataFigure resultFigure = beginProcessFigure.getResultFigure();

		ChainFlow chainFlow = modelStore.getChainFlowByUuid(id);

		FlowFigure flowFigure = new FlowFigure(resultFigure.pointR,
				endProcessFigure.pointL, resultFigure, endProcessFigure);
		flowFigure.setUuid(id);
		flowFigure.setModel(chainFlow);
		figureStore.addFlowFigure(flowFigure);

	}

	/**
	 * Creates process figure.
	 * @param figureStore
	 * 			figure container.
	 * @param processEle
	 * 			process element.
	 */
	@SuppressWarnings("unchecked")
	private void createProcessFigure(FigureStore figureStore, Element processEle) {
		String processUuid = processEle.attributeValue("uuid");
		String locXY = processEle.attributeValue("locXY");
		String[] locs = locXY.split(";");
		int x = Integer.parseInt(locs[0]);
		int y = Integer.parseInt(locs[1]);
		int width = Integer.parseInt(locs[2]);
		int heigh = Integer.parseInt(locs[3]);

		ProcessFigure processFigure = new ProcessFigure(x, y, width, heigh);
		AtomProcess atomModel = modelStore.getAtomProcessByUuid(processUuid);
		processFigure.setModel(atomModel);
		processFigure.setUuid(processUuid);
		figureStore.addProcessFigure(processFigure);

		String uuid, state;
		List<Element> dataEles = processEle.elements("DataFigure");
		for (Element dataEle : dataEles) {
			uuid = dataEle.attributeValue("uuid");
			state = dataEle.attributeValue("state");
			locXY = dataEle.attributeValue("locXY");
			locs = locXY.split(";");
			x = Integer.parseInt(locs[0]);
			y = Integer.parseInt(locs[1]);
			width = Integer.parseInt(locs[2]);
			heigh = Integer.parseInt(locs[3]);

			String paramName = uuid.split("#")[1];
			AbstractParam paramModel;
			if (paramName.equals("result")) {
				paramModel = atomModel.getOutPutParam();
			} else {
				paramModel = atomModel.getParamByName(paramName);
			}

			DataFigure dataFigure = new DataFigure(x, y, width, heigh);
			dataFigure.setUuid(uuid);
			dataFigure.setState(state);
			dataFigure.setModel(paramModel);
			processFigure.addDataFigure(dataFigure);

			if (state.equals("val")) {
				LinkFigure linkFigure = new LinkFigure(dataFigure.pointR,
						processFigure.pointL, dataFigure, processFigure);
				linkFigure.setUuid(uuid);
				processFigure.addLinkFigure(linkFigure);
				dataFigure.setLinkFigure(linkFigure);
			} else if (state.equals("res")) {
				LinkFigure linkFigure = new LinkFigure(processFigure.pointR,
						dataFigure.pointL, processFigure, dataFigure);
				processFigure.addLinkFigure(linkFigure);
				linkFigure.setUuid(uuid);
				dataFigure.setLinkFigure(linkFigure);
			}
		}
	}

}
