/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.model.util;

import java.io.File;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.ChainMeta;
import cn.edu.whu.lmars.model.ChainMeta.Provider;
import cn.edu.whu.lmars.model.ComplexParam;
import cn.edu.whu.lmars.model.LiteralParam;
import cn.edu.whu.lmars.model.OutPutParam;

/**
 * Creates models according the configuration file and adds them to a model
 * container. File extension is '.del'.
 * 
 * @author Mingda Zhang
 * 
 */

public class OpenModel {
	/**
	 * Gets ModelStore from configuration file.
	 * 
	 * @param path
	 *            the path configuration file
	 */
	@SuppressWarnings("unchecked")
	public PureModelStore open(String path) {
		if (!path.endsWith(".del")) {
			return null;
		}

		SAXReader reader = new SAXReader();
		try {
			PureModelStore modelStore = new PureModelStore();
			Document doc = reader.read(new File(path));
			Element root = doc.getRootElement();
			Element chainMetaEle = (Element) root.selectSingleNode("ChainMeta");
			ChainMeta chainMeta = createMeta(chainMetaEle);
			modelStore.setChainMeta(chainMeta);
			List<Element> atomList = root.selectNodes("AtomProcess");
			for (Element atom : atomList) {
				modelStore.addAtomProcess(createAtomProcess(atom));
			}

			List<Element> chains = root.elements("ChainFlow");
			for (Element chain : chains) {
				modelStore.addChainFlow(createChainFlow(chain));
			}
			return modelStore;
		} catch (DocumentException e) {
			e.printStackTrace();
			return null;
		}

	}

	/**
	 * Assigns value parsed from element to parameter.
	 * 
	 * @param param
	 *            parameter of a model.
	 * @param paramEle
	 *            parameter element.
	 */
	private void assignParamValue(AbstractParam param, Element paramEle) {
		String name = paramEle.attributeValue("name");
		String value = paramEle.attributeValue("value");
		String format = paramEle.attributeValue("format");
		param.setName(name);
		param.setFormat(format);
		param.setValue(value);
	}

	/**
	 * Creates flow model from flow element.
	 * 
	 * @param element
	 *            flow element.
	 * @return
	 */
	private ChainFlow createChainFlow(Element element) {
		ChainFlow chainFlow = new ChainFlow();
		chainFlow.setUuid(element.attributeValue("uuid"));
		return chainFlow;
	}

	/**
	 * Creates process model from process element.
	 * 
	 * @param element
	 *            process element.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private AtomProcess createAtomProcess(Element element) {
		AtomProcess atomProcess = new AtomProcess();
		String uuid = element.attributeValue("uuid");
		String name = element.attributeValue("name");
		String _abstract = element.attributeValue("abstract");
		atomProcess.setAbstract(_abstract);
		atomProcess.setName(name);
		atomProcess.setUUID(uuid);
		List<Element> literalEles = element.selectNodes("LiteralParam");
		for (Element literalEle : literalEles) {
			LiteralParam literalParam = new LiteralParam();
			assignParamValue(literalParam, literalEle);
			atomProcess.addLiteralParam(literalParam);
		}
		List<Element> complexEles = element.selectNodes("ComplexParam");

		for (Element compEle : complexEles) {
			ComplexParam complexParam = new ComplexParam();
			assignParamValue(complexParam, compEle);
			atomProcess.addComplexParam(complexParam);
		}

		Element outputEle = (Element) element.selectSingleNode("OutPutParam");
		OutPutParam outPutParam = new OutPutParam();
		assignParamValue(outPutParam, outputEle);
		atomProcess.setOutPutParam(outPutParam);
		return atomProcess;
	}

	/**
	 * Creates metadata from meta element.
	 * 
	 * @param element
	 *            metadata element.
	 * @return
	 */
	private ChainMeta createMeta(Element element) {
		String uuid = element.attributeValue("uuid");
		String name = element.attributeValue("name");
		String keyword = element.attributeValue("keyword");

		Element providerEle = (Element) element.selectSingleNode("Provider");
		String providerName = providerEle.attributeValue("name");
		String address = providerEle.attributeValue("address");

		ChainMeta chainMeta = new ChainMeta();
		Provider provider = chainMeta.new Provider();
		provider.setName(providerName);
		provider.setAddress(address);

		chainMeta.setProvider(provider);
		chainMeta.setUUID(uuid);
		chainMeta.setName(name);
		chainMeta.setKeyword(keyword);
		return chainMeta;
	}
}
