/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.model.util;

import java.util.ArrayList;
import java.util.List;

import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.ChainMeta;

/**
 * A model container.
 * 
 * @author Mingda Zhang
 * 
 */

public class PureModelStore {

	/** Metadata. */
	private ChainMeta chainMeta;
	/** Process Models */
	private List<AtomProcess> atomProcesses = new ArrayList<AtomProcess>();

	/** Flow Models. */
	private List<ChainFlow> chainFlows = new ArrayList<ChainFlow>();

	/**
	 * Gets Metadata.
	 */
	public ChainMeta getChainMeta() {
		return chainMeta;
	}

	/**
	 * Sets Metadata.
	 * @param value
	 * 			metadata.
	 */
	public void setChainMeta(ChainMeta value) {
		this.chainMeta = value;
	}

	/**
	 * Gets process models.
	 */
	public List<AtomProcess> getAtomProcesses() {
		return this.atomProcesses;
	}

	/**
	 * Gets flow models.
	 */
	public List<ChainFlow> getChainFlows() {
		return this.chainFlows;
	}

	/**
	 * Removes process model from the container.
	 */
	public void removeAtomProcess(AtomProcess atom) {
		if (atom == null) {
			return;
		}
		if (this.atomProcesses.contains(atom)) {
			this.atomProcesses.remove(atom);
		}
	}

	/**
	 * Removes flow model from the container.
	 * 
	 * @param flow
	 * 			flow model to be removed.
	 */
	public void removeChainFlow(ChainFlow flow) {
		if (flow == null) {
			return;
		}
		if (this.chainFlows.contains(flow)) {
			this.chainFlows.remove(flow);
		}
	}

	/**
	 * Adds process model to the container.
	 * 
	 * @param atom
	 * 			process model to be added.
	 */
	public void addAtomProcess(AtomProcess atom) {
		if (atom == null) {
			return;
		}
		if (!this.atomProcesses.contains(atom)) {
			this.atomProcesses.add(atom);
		}
	}

	/**
	 * Adds flow model to the container.
	 * 
	 * @param flow
	 * 			flow model to be added.
	 */
	public void addChainFlow(ChainFlow flow) {
		if (flow == null) {
			return;
		}
		if (!this.chainFlows.contains(flow)) {
			this.chainFlows.add(flow);
		}
	}

	/**
	 * Gets process model by id from the container.
	 * @param id
	 * 			the id of process model.
	 */
	public AtomProcess getAtomProcessByUuid(String id) {
		for (AtomProcess atomProcess : this.getAtomProcesses()) {
			if (atomProcess.getUUID().equals(id)) {
				return atomProcess;
			}
		}
		return null;
	}

	/**
	 * Removes process models by id from the container.
	 * 
	 * @param uuid
	 * 			the id of the process model.
	 */
	public void removeAtomProcess(String uuid) {
		this.removeAtomProcess(getAtomProcessByUuid(uuid));
	}

	/**
	 * Gets flow model by a front half portion of the id from the container.
	 * 
	 * @param uuid
	 * 			a front half portion of the id of the flow model.
	 * @return
	 */
	public ChainFlow getChainFlowByStartUuid(String uuid) {
		for (ChainFlow flow : this.getChainFlows()) {
			if (flow.getUuid().startsWith(uuid)) {
				return flow;
			}
		}
		return null;
	}

	/**
	 * Clears the container.
	 */
	public void clear() {
		this.getAtomProcesses().clear();
		this.getChainFlows().clear();
	}


	/**
	 * Gets flow model by id from the container.
	 */
	public ChainFlow getChainFlowByUuid(String uuid) {
		for (ChainFlow flow : this.getChainFlows()) {
			flow.getUuid().equals(uuid);
			return flow;
		}
		return null;
	}

	/**
	 * Sets process models.
	 * 
	 * @param atomProcesses
	 */
	public void setAtomProcesses(List<AtomProcess> atomProcesses) {
		this.atomProcesses = atomProcesses;
	}

	/**
	 * Sets flow models.
	 * 
	 * @param flows
	 */
	public void setChainFlows(List<ChainFlow> flows) {
		this.chainFlows = flows;
	}

}
