/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.model.util;

import java.io.FileWriter;
import java.io.IOException;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.ChainMeta;
import cn.edu.whu.lmars.model.ChainMeta.Provider;
import cn.edu.whu.lmars.model.ComplexParam;
import cn.edu.whu.lmars.model.LiteralParam;
import cn.edu.whu.lmars.utils.BaseTool;

/**
 * Saves models to file.
 * 
 * @author Mingda Zhang
 * 
 */
public class SaveModel {
	/**
	 * Saves models to file.
	 * 
	 * @param file
	 * 			the path of file.
	 * @param modelStore
	 * 			model container.
	 */
	public boolean saveToFile(String file, PureModelStore modelStore) {
		Document doc = DocumentHelper.createDocument();
		Element root = DocumentHelper.createElement("ComplexProcess");
		
		revertParams(modelStore);
		ChainMeta chainMeta = modelStore.getChainMeta();
		if (chainMeta == null) {
			chainMeta = new ChainMeta();
		}
		root.add(createMetaElement(chainMeta));
		for (AtomProcess atomProcess : modelStore.getAtomProcesses()) {
			root.add(createAtomProcessEle(atomProcess));
		}
		for (ChainFlow chainFlow : modelStore.getChainFlows()) {
			root.add(createFlowEle(chainFlow));
		}
		doc.setRootElement(root);

		FileWriter output;
		try {
			output = new FileWriter(file);
			output.write(doc.asXML());
			output.flush();
			output.close();
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}

		return true;
	}

	/**
	 * Creates meta element from metadata.
	 * @param chainMeta
	 * 			metadata.
	 * @return
	 */
	private Element createMetaElement(ChainMeta chainMeta) {
		Element metaEle = DocumentHelper.createElement("ChainMeta");
		String uuid = chainMeta.getUUID();
		String name = chainMeta.getName();
		String keyword = chainMeta.getKeyword();

		if (uuid == null) {
			uuid = "";
		}
		metaEle.addAttribute("uuid", uuid);
		if (name == null) {
			name = "";
		}
		metaEle.addAttribute("name", name);
		if (keyword == null) {
			keyword = "";
		}
		metaEle.addAttribute("keyword", keyword);
		Provider provider = chainMeta.getProvider();
		String address;
		Element providerEle = DocumentHelper.createElement("Provider");
		if (provider == null || provider.getAddress() == null) {
			address = "";
		} else {
			address = provider.getAddress();
		}
		providerEle.addAttribute("address", address);
		if (provider == null || provider.getName() == null) {
			name = "";
		} else {
			name = provider.getName();
		}
		providerEle.addAttribute("name", name);
		metaEle.add(providerEle);
		return metaEle;
	}

	/**
	 * Creates parameter element from the parameter of a model.
	 * @param paramType
	 * 			parameter type.
	 * @param param
	 * 			the parameter of a model.
	 * @return
	 */
	private Element createParamEle(String paramType, AbstractParam param) {
		Element paramEle = DocumentHelper.createElement(paramType);
		String name = param.getName();
		String format = param.getFormat();
		String value = param.getValue();

		if (name == null) {
			name = "";
		}
		paramEle.addAttribute("name", name);
		if (format == null) {
			format = "";
		}
		paramEle.addAttribute("format", format);
		if (value == null) {
			value = "";
		}
		paramEle.addAttribute("value", value);

		return paramEle;
	}

	/**
	 * Creates process model from process element.
	 * @param processModel
	 * 			process model
	 * @return
	 */
	private Element createAtomProcessEle(AtomProcess processModel) {
		String uuid = processModel.getUUID();
		String name = processModel.getName();
		String _abstract = processModel.getAbstract();

		Element atomEle = DocumentHelper.createElement("AtomProcess");
		if (uuid == null) {
			uuid = "";
		}
		atomEle.addAttribute("uuid", uuid);

		if (name == null) {
			name = "";
		}
		atomEle.addAttribute("name", name);

		if (_abstract == null) {
			_abstract = "";
		}
		atomEle.addAttribute("abstract", _abstract);

		for (ComplexParam param : processModel.getComplexParam()) {
			atomEle.add(createParamEle("ComplexParam", param));
		}
		for (LiteralParam param : processModel.getLiteralParam()) {
			atomEle.add(createParamEle("LiteralParam", param));
		}
		atomEle.add(createParamEle("OutPutParam", processModel.getOutPutParam()));
		return atomEle;
	}

	/**
	 * Creates flow element from flow model.
	 * 
	 * @param flowModel
	 *            flow model.
	 */
	private Element createFlowEle(ChainFlow flowModel) {
		Element flowEle = DocumentHelper.createElement("ChainFlow");
		String uuid = flowModel.getUuid();
		flowEle.addAttribute("uuid", uuid);
		return flowEle;
	}

	/**
	 * Restores parameter values to maintain the independence of the model.
	 */
	private void revertParams(PureModelStore modelStore){
		for(ChainFlow flow:modelStore.getChainFlows()){
			String id = flow.getUuid();
			String[] ids = id.split("#");
			if (ids.length<3) {
				continue;
			}
			AtomProcess beginAtom = modelStore.getAtomProcessByUuid(ids[0]);
			AtomProcess endAtom = modelStore.getAtomProcessByUuid(ids[1]);
			AbstractParam param = endAtom.getParamByName(ids[2]);
			AbstractParam outPutParam = beginAtom.getOutPutParam();
			String value = outPutParam.getValue();
			if (!BaseTool.isNull(value)) {
				outPutParam.setValue("");
			}
			value = outPutParam.getFormat();
			if (!BaseTool.isNull(value)) {
				outPutParam.setFormat("");
			}
			String paramID = ids[0]+"#result";
			if (!param.getValue().equals(paramID)) {
				param.setValue(paramID);
			}
		}
	}
}
