/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.beans;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.run.bind.beans.BindProcess;
import cn.edu.whu.lmars.run.bind.beans.BindStore;

/**
 * Parses models to a tree.
 * 
 * @author Mingda Zhang
 * 
 */
public class ProcessModelTree {
	private PureModelStore modelStore;
	private BindStore bindStore;
	private List<ModelTreeNode> treeNodes = new ArrayList<ModelTreeNode>();
	private List<ModelTreeNode> heads = new ArrayList<ModelTreeNode>();
	private List<ModelTreeNode> tails = new ArrayList<ModelTreeNode>();
	private Logger logger = Logger.getLogger(ProcessModelTree.class);
	private StringBuffer err;
	private boolean finished = true;
	public ProcessModelTree(PureModelStore modelStore, BindStore bindStore,
			StringBuffer err) {
		this.modelStore = modelStore;
		this.err = err;
		this.bindStore = bindStore;
		logger.info("create model tree");
	}

	/**
	 * Gets root of the tree.
	 * 
	 * @return
	 */
	public ModelTreeNode getHead() {
		return heads.get(0);
	}

	/**
	 * Generates a tree.
	 * 
	 * @return
	 */
	public boolean generateTree() {
		boolean flag = createTreeNodes();
		if (!flag) {
			return false;
		}
		if (treeNodes.size() == 1) {
			heads.add(treeNodes.get(0));
			return true;
		}
		String startID, endID, flowID;
		ModelTreeNode startTreeNode = null, endTreeNode = null;
		for (ChainFlow chainFlow : modelStore.getChainFlows()) {

			flowID = chainFlow.getUuid();
			String[] flows = flowID.split("#");
			if (flows.length < 2) {
				logger.error("failed to parse flow id");
				err.append("failed to parse flowID:" + flowID
						+ ",the right format is Handle1_ID#Handle2_ID");
				return false;
			}
			startID = flows[0];
			endID = flows[1];
			startTreeNode = getNodeById(startID);
			endTreeNode = getNodeById(endID);
			if (startTreeNode == null || endTreeNode == null) {
				String errStr = "Failed to parse model, can't get AtomProcess: id:"
						+ flowID;
				logger.error(errStr);
				err.append(errStr);
				return false;
			}
			addHead(endTreeNode);
			addTail(startTreeNode);
			endTreeNode.addChild(startTreeNode);
		}

		for (ModelTreeNode tail : tails) {
			removeHead(tail);
		}
		int num = heads.size();
		if (num != 1) {
			err.append("Root should be just one.");
			return false;
		}
		return true;
	}

	/**
	 * Checks whether all parameters in model container are assigned.
	 * 
	 * @return
	 */
	public boolean isFinished() {
		// AtomProcess atomProcess;
		/*String value;
		for (AtomProcess atomProcess : modelStore.getAtomProcesses()) {
			for (ComplexParam complexParam : atomProcess.getComplexParam()) {
				value = complexParam.getValue();
				if (BaseTool.isNull(value)) {
					err.append(atomProcess.getName() + "@"
							+ complexParam.getName() + " is null" + "\n");
					return false;
				}
			}

			for (LiteralParam literalParam : atomProcess.getLiteralParam()) {
				value = literalParam.getValue();
				if (BaseTool.isNull(value)) {
					err.append(atomProcess.getName() + "@"
							+ literalParam.getName() + " is null" + "\n");
					return false;
				}
			}

		}
		return true;*/
		return checkParameters(getHead());
	}

	/**
	 * Removes a tree node from head list.
	 * 
	 * @param head
	 */
	private void removeHead(ModelTreeNode head) {
		if (heads.contains(head)) {
			heads.remove(head);
		}
	}

	/**
	 * Adds a tree node to the head list.
	 * 
	 * @param head
	 *            tree node.
	 */
	private void addHead(ModelTreeNode head) {
		if (!heads.contains(head)) {
			heads.add(head);
		}
	}

	/**
	 * Adds a tree node to the tail list.
	 * 
	 * @param tail
	 *            tree node.
	 */
	private void addTail(ModelTreeNode tail) {
		if (!tails.contains(tail)) {
			tails.add(tail);
		}
	}

	/**
	 * Creates tree node.
	 */
	private boolean createTreeNodes() {
		treeNodes.clear();
		BindProcess bindProcess;
		String atomId;
		for (AtomProcess atom : modelStore.getAtomProcesses()) {
			atomId = atom.getUUID();
			bindProcess = bindStore.getBindProcess(atomId);
			if (bindProcess == null || bindProcess.getCurBindAtom() == null) {
				err.append(atom.getName() + " has no binding information ");
				return false;
			}
			ModelTreeNode treeNode = new ModelTreeNode(atom,
					bindProcess.getCurBindAtom());
			treeNodes.add(treeNode);
		}
		return true;
	}

	/**
	 * Gets a tree node by id.
	 * 
	 * @param id
	 *            the id of tree node.
	 * @return
	 */
	private ModelTreeNode getNodeById(String id) {
		for (ModelTreeNode node : treeNodes) {
			if (node.getSelf().getUUID().equals(id)) {
				return node;
			}
		}
		return null;
	}

	private boolean checkParameters(ModelTreeNode modelNode){
		if (!finished) {
			return false;
		}
		int num = modelNode.getChildren().size();
		if (num == 0) {
			return modelNode.isReady();
		}
		for (ModelTreeNode child : modelNode.getChildren()) {
			/*if (!finished) {
				return false;
			}*/
			finished = checkParameters(child);
		}
		return modelNode.isReady();
	
	}
	/**
	 * Gets err.
	 * 
	 * @return erro information.
	 */
	public String getErr() {
		return err.toString();
	}
}
