/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.beans;

import java.util.Map;

import org.apache.log4j.Logger;

import conf.GlobeParam;

import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.run.bind.beans.BindStore;
import cn.edu.whu.lmars.run.ui.ExecuteMonitorDialog;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.HandleTool;

/**
 * Execution class. Controls execution sequence.
 * 
 * @author Mingda Zhang
 * 
 */
public class RunModelTree {
	/** a combination of services to be executed. */
	private ProcessModelTree modelTree = null;
	/** logger. */
	private Logger logger = Logger.getLogger(RunModelTree.class);
	/** error information. */
	private StringBuffer err = new StringBuffer();
	/** If false, out of the composite service execution. */
	private boolean exe_flag = true;
	/** model container. */
	private PureModelStore modelStore;
	/** service execution sequence. */
	private int seqNum = 1;
	/**
	 * figure container, the state of figure will be changed during the process
	 * of the composite service execution.
	 */
	private FigureStore figureStore;
	/** Execution monitor dialog. */
	private ExecuteMonitorDialog monitorDialog = ExecuteMonitorDialog
			.getInstance(GlobeParam.getClient());

	public RunModelTree(PureModelStore modelStore, FigureStore figureStore,
			BindStore bindStore) {
		modelTree = new ProcessModelTree(modelStore, bindStore, err);
		this.modelStore = modelStore;
		this.figureStore = figureStore;
	}

	/**
	 * Checks whether the combination of services to meet the execution
	 * requirements.
	 * 
	 * @return true if the composite service meets the requirements.
	 */
	public boolean check() {
	//	monitorDialog.appendMsg("Ready to execute\n");
		monitorDialog.appendMsg("Checking models...\n");
		boolean isTree = modelTree.generateTree();
		if (!isTree) {
			monitorDialog.appendMsg("Error occured, not a complete service chain.\n");
			return false;
		}
		monitorDialog.appendMsg("Checking parameters...\n");
		boolean isFished = modelTree.isFinished();
		if (!isFished) {
			monitorDialog.appendMsg("Please assign all required parameters.\n");
			return false;
		}
		return true;
	}

	/**
	 * Executes the composite service.
	 * 
	 * @return true if composite service was executed successfully.
	 */
	public boolean execute() {
		if (!check()) {
			return false;
		}
		monitorDialog.appendMsg("Begin to execute\n");
		logger.info("Begin to execute\n");
		ModelTreeNode head = modelTree.getHead();
		if (head == null) {
			logger.error("No entry\n");
			err.append("No enty\n");
			return false;
		}
		return run(head);
	}

	/**
	 * Parses the service chain into a tree. Firstly executes the leaf nodes and
	 * then runs a parent node.
	 * 
	 * @param head
	 *            the root of the tree.
	 */
	private boolean run(ModelTreeNode head) {
		if (!exe_flag) {
			return false;
		}
		int num = head.getChildren().size();
		if (num == 0) {
			head.setSequence(seqNum++);
			exe_flag = head.execute();
			changeState(head, exe_flag);
			return exe_flag;
		}
		for (ModelTreeNode child : head.getChildren()) {
			/*if (!exe_flag) {
				return false;
			}*/
			run(child);
		}
		head.setSequence(seqNum++);
		if (!exe_flag) {
			return false;
		}

		exe_flag = head.execute();

		changeState(head, exe_flag);
		return exe_flag;
	}

	/**
	 * Gets err.
	 * 
	 * @return the error information during the process of execution.
	 */
	public String getErr() {
		return err.toString();
	}

	/**
	 * Changes the state of figure according to the implementation of the
	 * service and assigns the result value to the parameter of next service.
	 * 
	 * @param treeNode
	 *            a tree node in the service chain.
	 * @param flag
	 */
	private void changeState(ModelTreeNode treeNode, boolean flag) {
		String id = treeNode.getSelf().getUUID();
		AtomProcess fromAtomProcess = treeNode.getSelf();
		ProcessFigure figure = figureStore.getProcessFigureByUuid(id);
		Map<String, String> resultMap = treeNode.getExecutionInfo().getOutputMap();
		if (resultMap == null) {
			exe_flag = false;
			figure.setExecuteState(ExecutionState.FAILED);
			HandleTool.refreshFigShow();
			return;
		}
		String value = resultMap.get("result_value");
		String format = resultMap.get("result_datatype");
		if (flag) {
			if (BaseTool.isNull(value)) {
				exe_flag = false;
				err.append(fromAtomProcess.getName() + " has no result");
				figure.setExecuteState(ExecutionState.FAILED);
				HandleTool.refreshFigShow();
				return;
			} else {
				AbstractParam resultParam = fromAtomProcess.getOutPutParam();
				resultParam.setValue(value);
				resultParam.setFormat(format == null ? "" : format);

				ChainFlow chainFlow = modelStore.getChainFlowByStartUuid(id);
				if (chainFlow != null) {
					String[] ids = chainFlow.getUuid().split("#");
					AtomProcess toAtom = modelStore
							.getAtomProcessByUuid(ids[1]);
					AbstractParam param = toAtom.getParamByName(ids[2]);
					param.setValue(value);
					if (!BaseTool.isNull(format)) {
						param.setFormat(format);
					}
				}
				figure.setExecuteState(ExecutionState.FINISHED);
				figure.setExecutionInfo(treeNode.getExecutionInfo());
			}
		} else {
			figure.setExecuteState(ExecutionState.FAILED);
		}
		HandleTool.refreshFigShow();
	}
}
