/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.bind.utils;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.run.bind.beans.BindAtom;
import cn.edu.whu.lmars.run.bind.beans.BindAtom.ParamMapInfo;
import cn.edu.whu.lmars.run.bind.beans.BindProcess;
import cn.edu.whu.lmars.run.bind.beans.BindStore;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.Dom4jXMLTool;

/**
 * Binding tool.Saves binding information to file and gets binding information
 * from file.
 * 
 * @author Mingda Zhang
 * 
 */
public class BindTool {

	/**
	 * Creates a binding container from file.
	 * 
	 * @param file
	 *            the file stores binding information.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static BindStore initBindStore(File file) {
		SAXReader reader = new SAXReader();
		BindStore bindStore = new BindStore();
		try {
			Document doc = reader.read(file);
			Element root = doc.getRootElement();
			List<Element> processEleList = root.elements("Process");
			for (Element processEle : processEleList) {
				bindStore.addBindProcess(parseBindProcess(processEle));
			}
		} catch (DocumentException e) {
			e.printStackTrace();
		}
		return bindStore;
	}

	/**
	 * Saves binding information to a file.
	 * 
	 * @param bindStore
	 *            binding container.
	 * @param file
	 *            File to save binding information.
	 * @return
	 */
	public static boolean saveBindStore(PureModelStore modelStore,BindStore bindStore, File file) {
		Document doc = DocumentHelper.createDocument();
		Element root = DocumentHelper.createElement("Mapping");

		for (BindProcess process : bindStore.getBindProcessList()) {
			if (modelStore.getAtomProcessByUuid(process.getModelID()) == null) {
				continue;
			}
			root.add(saveBindProcess(process));
		}
		doc.setRootElement(root);

		String formatXml = Dom4jXMLTool.format(doc.asXML());
		if (formatXml == null) {
			formatXml = doc.asXML();
		}

		FileWriter output;
		try {
			output = new FileWriter(file);
			output.write(formatXml);
			output.flush();
			output.close();
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	/**
	 * Gets process binding from process element.
	 * 
	 * @param processEle
	 *            process element.
	 * @return process binding.
	 */
	@SuppressWarnings("unchecked")
	private static BindProcess parseBindProcess(Element processEle) {
		BindProcess bindProcess = new BindProcess();
		String id = processEle.attributeValue("id");
		String curID = processEle.attributeValue("curId");
		bindProcess.setModelID(id);
		bindProcess.setCurMapID(curID);

		List<Element> atomEleList = processEle.elements("Map");
		for (Element atomEle : atomEleList) {
			bindProcess.addMapInfo(parseBindAtom(atomEle));
		}
		return bindProcess;
	}

	/**
	 * Gets a process binding from element.
	 * 
	 * @param atomEle
	 * @return
	 */
	private static BindAtom parseBindAtom(Element atomEle) {
		BindAtom bindAtom = new BindAtom();
		String id = atomEle.attributeValue("id");
		String name = atomEle.attributeValue("name");
		String serviceUrl = atomEle.elementText("ServiceUrl");
		String serviceType = atomEle.elementText("ServiceType");
		String version = atomEle.elementText("Version");
		String operationName = atomEle.elementText("Operation");
		bindAtom.setId(id);
		bindAtom.setName(name);
		bindAtom.setUrl(serviceUrl);
		bindAtom.setServiceType(BaseTool.getServiceTypeFromString(serviceType));
		bindAtom.setVersion(version);
		bindAtom.setOperationName(operationName);
		@SuppressWarnings("unchecked")
		List<Element> paramEleList = atomEle.elements("ParamMap");
		for (Element paramEle : paramEleList) {
			ParamMapInfo paramMapInfo = bindAtom.new ParamMapInfo();
			paramMapInfo.setFrom(paramEle.elementText("From"));
			paramMapInfo.setTo(paramEle.elementText("To"));
			bindAtom.addParamMapInfo(paramMapInfo);
		}
		return bindAtom;
	}

	/**
	 * Gets an element from a process binding.
	 * 
	 * @param bindAtom
	 * @return
	 */
	private static Element saveBindAtom(BindAtom bindAtom) {
		Element atomEle = DocumentHelper.createElement("Map");
		atomEle.addAttribute("id", bindAtom.getId());
		atomEle.addAttribute("name", bindAtom.getName());
		atomEle.addElement("ServiceUrl").setText(bindAtom.getUrl());
		atomEle.addElement("ServiceType").setText(
				bindAtom.getServiceType().toString());
		atomEle.addElement("Version").setText(bindAtom.getVersion());
		atomEle.addElement("Operation").setText(bindAtom.getOperationName());
		for (ParamMapInfo paramMapInfo : bindAtom.getParamMapInfos()) {
			Element paramEle = atomEle.addElement("ParamMap");
			paramEle.addElement("From").setText(paramMapInfo.getFrom());
			paramEle.addElement("To").setText(paramMapInfo.getTo());
		}
		return atomEle;
	}

	/**
	 * Gets an element from process bindings.
	 * 
	 * @param bindProcess
	 * @return
	 */
	private static Element saveBindProcess(BindProcess bindProcess) {
		Element processEle = DocumentHelper.createElement("Process");
		processEle.addAttribute("id", bindProcess.getModelID());
		processEle.addAttribute("curId", bindProcess.getCurMapID());
		for (BindAtom atom : bindProcess.getMapInfos()) {
			processEle.add(saveBindAtom(atom));
		}
		return processEle;
	}

}
