/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.sensorweb.sos.utils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.Node;
import org.dom4j.QName;
import org.dom4j.XPath;

import cn.edu.whu.lmars.sensorweb.sos.SOSConf;
import cn.edu.whu.lmars.sensorweb.sos.adapter.SOSAdapter;
import cn.edu.whu.lmars.sensorweb.sos.beans.SOSSensor;
import cn.edu.whu.lmars.sensorweb.sos.beans.SensorPosition;
import cn.edu.whu.lmars.service.adapter.AdapterFactory;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IParameterInfo;
import cn.edu.whu.lmars.service.beans.IService;
import cn.edu.whu.lmars.service.beans.ServiceModel;
import cn.edu.whu.lmars.service.beans.ServiceType;
import cn.edu.whu.lmars.service.utils.RequestMethodStore;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.Dom4jXMLTool;
import cn.edu.whu.lmars.utils.FileTool;
import conf.PathConf;

/**
 * Sensor operation.
 * 
 * @author Mingda Zhang
 * 
 */
public class SensorUtility {
	/**
	 * Converts a sensor to html.
	 * 
	 * @param sensorBean
	 * @return
	 */
	public static String sensorToHtml(SOSSensor sensorBean) {
		Element htmlEle = DocumentHelper.createElement("html");
		Element headEle = htmlEle.addElement("head");
		headEle.setText("Procedure Description");
		Element bodyEle = htmlEle.addElement("body");
		Element tableEle = bodyEle.addElement("table");
		String value = null;

		value = sensorBean.getProcedure();
		if (!BaseTool.isNull(value)) {
			tableEle.add(createTrEle("procedure:", value));
		}

		value = sensorBean.getUrl();
		if (!BaseTool.isNull(value)) {
			tableEle.add(createTrEle("sos_url:", value));
		}
		value = sensorBean.getVersion();
		if (!BaseTool.isNull(value)) {
			tableEle.add(createTrEle("sos_version:", value));
		}
		SensorPosition sensorPosition = sensorBean.getSensorPosition();
		double degree;
		if (sensorPosition != null) {
			degree = sensorPosition.getNorthing();
			if (!BaseTool.isNull(degree)) {
				value = sensorPosition.getNorthingUom();
				value = degree + "  " + value;
				tableEle.add(createTrEle("latitude:", value));
			}
			degree = sensorPosition.getEasting();
			if (!BaseTool.isNull(degree)) {
				value = sensorPosition.getEastingUom();
				value = degree + "  " + value;
				tableEle.add(createTrEle("longitude:", value));
			}
			degree = sensorPosition.getAltitude();
			if (!BaseTool.isNull(degree)) {
				value = sensorPosition.getAltitudeUom();
				value = degree + "  " + value;
				tableEle.add(createTrEle("altitude:", value));
			}
		}
		value = sensorBean.getOffer();
		if (!BaseTool.isNull(value)) {
			tableEle.add(createTrEle("offer:", value));
		}

		List<String> observedProperties = sensorBean.getObservedProperties();
		if (observedProperties != null && observedProperties.size() > 0) {
			value = "";
			for (String observedProperty : observedProperties) {
				value += observedProperty + ";";
			}
			tableEle.add(createTrEle("observedProperty:", value));
		}
		return htmlEle.asXML();
	}

	/**
	 * Creates a line element.
	 * 
	 * @param name
	 * @param value
	 * @return
	 */
	private static Element createTrEle(String name, String value) {
		Element trEle = DocumentHelper.createElement("tr");
		trEle.addElement("td").setText(name);
		trEle.addElement("td").setText(value);
		return trEle;
	}

	/**
	 * Gets a sensor from specified service.
	 * 
	 * @param service
	 *            SOS service
	 * @param sensorName
	 *            sensor name.
	 * @return
	 */

	@SuppressWarnings("unchecked")
	public static SOSSensor parseSensor(IService service, String sensorName) {
		IAdapter adapter = AdapterFactory.getInstance().getAdapter(service);

		SOSSensor sensorBean = new SOSSensor();
		sensorBean.setUrl(service.getUrl());
		sensorBean.setVersion(service.getVersion());
		sensorBean.setProcedure(sensorName);
		IOperationInfo operationInfo = adapter.getOperationInfo(sensorName);
		IParameterInfo offerParameterInfo = operationInfo
				.getIParameterInfo(SOSConf.Offering);
		String offer = offerParameterInfo.getMeta().get_allowValue();
		if (!BaseTool.isNull(offer)) {
			sensorBean.setOffer(offer);
		}
		IParameterInfo observedParameterInfo = operationInfo
				.getIParameterInfo(SOSConf.ObservedProperty);
		String observedProperty = observedParameterInfo.getMeta()
				.get_allowValue();
		if (!BaseTool.isNull(observedProperty)) {
			for (String obs : observedProperty.split(";")) {
				if (!BaseTool.isNull(obs)) {
					sensorBean.addObservedProperty(obs);
				}
			}
		}
		try {
			Namespace smlNs = new Namespace("sml",
					"http://www.opengis.net/sensorML/1.0.1");
			Namespace sweNs = new Namespace("swe",
					"http://www.opengis.net/swe/1.0.1");
			Namespace sosNs = new Namespace("sos",
					"http://www.opengis.net/sos/1.0");
			QName quantityQN = new QName("Quantity", sweNs);
			QName valueQN = new QName("value", sweNs);
			QName uomQN = new QName("uom", sweNs);
			Map<String, String> map = new HashMap<String, String>();
			map.put(smlNs.getPrefix(), smlNs.getURI());
			map.put(sweNs.getPrefix(), sweNs.getURI());
			map.put(sosNs.getPrefix(), sosNs.getURI());
			Document describeDoc;
			if (adapter instanceof SOSAdapter) {
				describeDoc = ((SOSAdapter) adapter).getDescribeDoc(sensorName);
			} else {
				Document doc = FileTool
						.getDocFromResource(PathConf.SOS_Describe_Sensor);

				XPath procedureXPath = DocumentHelper
						.createXPath("//sos:procedure");
				procedureXPath.setNamespaceURIs(map);
				Node procedureNode = procedureXPath.selectSingleNode(doc);
				if (procedureNode == null) {
					return sensorBean;
				}
				procedureNode.setText(sensorName);
				describeDoc = RequestMethodStore.POSTDoc(doc.asXML(), adapter
						.getServiceInfo().getDescribeURI());
			}

			String mobileXpathExp = "//sml:capabilities/swe:SimpleDataRecord/swe:field[@name='mobile']/swe:Boolean/swe:value";

			Node mobileNode = Dom4jXMLTool.selectSingleXmlNode(map,
					mobileXpathExp, describeDoc);
			if (mobileNode == null) {
				return sensorBean;
			}
			String flag = mobileNode.getText();
			boolean mobileAble = Boolean.parseBoolean(flag);
			sensorBean.setMobile(mobileAble);
			if (!mobileAble) {
				String sensorPositionXPath = "//sml:position/swe:Position/swe:location/swe:Vector/swe:coordinate";
				SensorPosition sensorPosition = new SensorPosition();
				List<Element> coordinateNodes = Dom4jXMLTool.selectXmlNodes(
						map, sensorPositionXPath, describeDoc);
				if (coordinateNodes == null || coordinateNodes.size() == 0) {
					return sensorBean;
				}

				// Node valueNode,degreeNode;
				for (Element element : coordinateNodes) {
					String name, uom = null;
					Element quanEle = element.element(quantityQN);
					if (quanEle == null) {
						continue;
					}
					Element valueNode = quanEle.element(valueQN);
					if (valueNode == null) {
						continue;
					}
					String value = valueNode.getText();
					Element degreeNode = quanEle.element(uomQN);
					if (degreeNode != null) {
						uom = degreeNode.attributeValue("code");
					}
					name = element.attributeValue("name");
					if (BaseTool.isNull(name)) {
						continue;
					}
					if (name.equalsIgnoreCase("longitude")
							|| name.equalsIgnoreCase("easting")) {
						sensorPosition.setEasting(Double.parseDouble(value));
						sensorPosition.setEastingUom(uom);
					} else if (name.equalsIgnoreCase("northing")
							|| name.equalsIgnoreCase("latitude")) {
						sensorPosition.setNorthing(Double.parseDouble(value));
						sensorPosition.setNorthingUom(uom);
					} else if (name.equalsIgnoreCase("altitude")) {
						sensorPosition.setAltitude(Double.parseDouble(value));
						sensorPosition.setAltitudeUom(uom);
					}
				}
				sensorBean.setSensorPosition(sensorPosition);
			}
		} catch (Exception e) {
		}

		return sensorBean;
	}

	public static void main(String[] args) {
		ServiceModel service = new ServiceModel();
		service.setServiceType(ServiceType.SOS);
		service.setUrl("http://localhost:8081/52nSOSv3/sos");
		service.setVersion("1.0.0");
		String sensorName = "urn:ogc:object:feature:Sensor:IFGI:ifgi-sensor-1";
		SOSSensor sensorBean = parseSensor(service, sensorName);
		System.out.println(sensorBean);
	}
}
