/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.tree.utils;

import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragSource;
import java.io.File;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellEditor;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.service.beans.ServiceType;
import cn.edu.whu.lmars.tree.listeners.dragdrop.GeoDragGestureListener;
import cn.edu.whu.lmars.tree.nodes.ServiceTreeNode;
import cn.edu.whu.lmars.tree.nodes.TreeNodeImpl;
import cn.edu.whu.lmars.utils.BaseTool;
import conf.PathConf;

/**
 * Create a tree from a configuration file of service.
 * 
 * @author Mingda Zhang
 * 
 */
public class TreeCreate {

	/** the location of configure file of the tree */
	private String treeConfPath = null;
	private String url = "url";
	private String version = "version";
	private String name = "name";
	private String id = "id";
	private Logger logger = Logger.getLogger(TreeCreate.class);

	public TreeCreate() {
		this.treeConfPath = PathConf.getServicConfDir();
	}

	/**
	 * Executes the creating tree operation.
	 * 
	 * @return javax.swing.JTree;
	 */
	@SuppressWarnings("unchecked")
	public JTree execute() {
		if (BaseTool.isNull(treeConfPath)) {
			logger.info("Can't open the the configure file of tree");
		}

		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("Node");
		JTree tree = new JTree(rootNode);
		tree.setVisible(true);
		tree.setEditable(false);
		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);

		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		try {
			renderer.setOpenIcon(new ImageIcon(getClass().getResource(
					"/images/parent.png")));
			renderer.setLeafIcon(new ImageIcon(getClass().getResource(
					"/images/child.png")));
			renderer.setClosedIcon(new ImageIcon(getClass().getResource(
					"/images/parent.png")));
		} catch (Exception e) {
			logger.error("Not found ImageIcon");
		}
		tree.setCellRenderer(renderer);

		tree.setCellEditor(new DefaultTreeCellEditor(tree,
				new DefaultTreeCellRenderer()));

		DragSource dragSource = DragSource.getDefaultDragSource();

		dragSource.createDefaultDragGestureRecognizer(tree,
				DnDConstants.ACTION_COPY_OR_MOVE, new GeoDragGestureListener());

		// create the parent tree node
		TreeNodeImpl wpsNode = createTreeNode("Geoprocessing",
				ServiceType.GeoProcessing);
		TreeNodeImpl dataNode = createTreeNode("Web Datas", ServiceType.WebData);
		TreeNodeImpl sensorWebNode = createTreeNode("Sensor Web",
				ServiceType.SensorWeb);
		rootNode.add(wpsNode);
		rootNode.add(dataNode);
		rootNode.add(sensorWebNode);

		// parse the configure file
		try {
			SAXReader reader = new SAXReader();
			Document doc = reader.read(new File(treeConfPath));
			List<Element> wpsEles = doc.selectNodes("//WPSService");
			for (Element wpsEle : wpsEles) {
				wpsNode.add(createServiceNode(wpsEle, ServiceType.WPS));
			}
			List<Element> wfsEles = doc.selectNodes("//WFSService");
			for (Element wfsEle : wfsEles) {
				dataNode.add(createServiceNode(wfsEle, ServiceType.WFS));
			}
			List<Element> wcsEles = doc.selectNodes("//WCSService");
			for (Element wcsEle : wcsEles) {
				dataNode.add(createServiceNode(wcsEle, ServiceType.WCS));
			}

			List<Element> sosEles = doc.selectNodes("//SOSService");
			for (Element sosEle : sosEles) {
				sensorWebNode.add(createServiceNode(sosEle, ServiceType.SOS));
			}

			List<Element> sesEles = doc.selectNodes("//SESService");
			for (Element sesEle : sesEles) {
				sensorWebNode.add(createServiceNode(sesEle, ServiceType.SES));
			}

			List<Element> wnsEles = doc.selectNodes("//WNSService");
			for (Element wnsEle : wnsEles) {
				sensorWebNode.add(createServiceNode(wnsEle, ServiceType.SES));
			}
		} catch (Exception e) {
			logger.error("There an exception when reading configure file of the tree:\n"
					+ e.getMessage());
		}

		
		DefaultTreeModel model = (DefaultTreeModel) tree.getModel();
		TreePath path = new TreePath(model.getPathToRoot(wpsNode));
		tree.expandPath(path);
		tree.addSelectionPath(path);
		tree.scrollPathToVisible(path);
		
		
		return tree;
	}

	/**
	 * Creates a tree node.
	 * 
	 * @param name
	 *            the name of tree node
	 * @param nodeType
	 *            the type of tree node
	 * @return cn.edu.whu.lmars.tree.nodes.TreeNodeImpl
	 */
	private TreeNodeImpl createTreeNode(String name, ServiceType nodeType) {
		TreeNodeImpl treeNode = new TreeNodeImpl(name);
		treeNode.setServiceType(nodeType);
		return treeNode;
	}

	/**
	 * Creates a Service tree node.
	 * 
	 * @param element
	 *            the element to parse
	 * @param childPath
	 *            the location of configure content
	 * @param nodeType
	 *            the type of tree node
	 * @return cn.edu.whu.lmars.tree.nodes.ServiceTreeNode
	 */
	@SuppressWarnings("unchecked")
	private ServiceTreeNode createServiceNode(Element element,
			ServiceType nodeType) {
		String url = element.attributeValue(this.url);
		String name = element.attributeValue(this.name);
		String version = element.attributeValue(this.version);
		String id = element.attributeValue(this.id);
		ServiceTreeNode serviceNode = new ServiceTreeNode(name);
		serviceNode.setServiceType(nodeType);
		serviceNode.setUrl(url);
		serviceNode.setVersion(version);
		serviceNode.setId(id);
		List<Element> list = element.elements();
		for (Element ele : list) {
			name = ele.getTextTrim();
			TreeNodeImpl treeNode = new TreeNodeImpl(name);
			treeNode.setServiceType(nodeType);
			serviceNode.add(treeNode);
		}
		// Initializes the adapter when application starts.
	//	AdapterFactory.getInstance().getAdapter(serviceNode);
		return serviceNode;
	}
}
