/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.utils;

import gov.nasa.worldwind.WorldWindow;
import gov.nasa.worldwind.layers.Layer;
import gov.nasa.worldwind.layers.LayerList;
import gov.nasa.worldwind.layers.placename.PlaceNameLayer;
import gov.nasa.worldwind.util.WWIO;
import gov.nasa.worldwind.util.gdal.GDALUtils;

import java.awt.Component;
import java.io.File;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.JViewport;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.figure.utils.FigureUtility;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ComplexParam;
import cn.edu.whu.lmars.model.OutPutParam;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.sensorweb.sos.beans.SOSSensor;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.FileTool;
import cn.edu.whu.lmars.worldwind.balloon.SensorBalloon;
import cn.edu.whu.lmars.worldwind.datatree.DataLayerInfo;
import cn.edu.whu.lmars.worldwind.datatree.DataLayerPanel;
import cn.edu.whu.lmars.worldwind.datatree.DataTreeNode;
import cn.edu.whu.lmars.worldwind.formats.FileLoadFactory;
import cn.edu.whu.lmars.worldwind.formats.IFileLoad;
import cn.edu.whu.lmars.worldwind.ui.DownloadMonitorDialog;
import cn.edu.whu.lmars.worldwind.ui.WWDPanel;
import conf.GlobeParam;
import conf.PathConf;

/**
 * Utility class handles World Wind operation.
 * 
 * @author Mingda Zhang
 * 
 */
public class WWDUtility {
	private static Logger logger = Logger.getLogger(WWDUtility.class);
	static {
		@SuppressWarnings("unused")
		GDALUtils gdalUtil = new GDALUtils();
	}

	/**
	 * Inserts the layer into the layer list just before the placename layer.
	 * 
	 * @param wwd
	 * @param layer
	 */
	public static void insertBeforePlacenames(WorldWindow wwd, Layer layer) {
		int compassPosition = 0;
		LayerList layers = wwd.getModel().getLayers();
		for (Layer l : layers) {
			if (l instanceof PlaceNameLayer)
				compassPosition = layers.indexOf(l);
		}
		layers.add(compassPosition, layer);
	}

	/**
	 * Inserts a data layer to the WorldWindow.
	 * 
	 * @param wwd
	 *            WorldWindow
	 * @param layer
	 *            layer to be added.
	 * @param dataNode
	 *            data node, data information.
	 */
	public static void insertDataLayer(WorldWindow wwd, Layer layer,
			DataLayerInfo dataNode) {
		LayerList layers = wwd.getModel().getLayers();
		layers.add(layer);
		DataTreeNode dataTreeNode = new DataTreeNode(dataNode, layer);
		DataLayerPanel layerPanel = GlobeParam.getDataLayerPanel();
		layerPanel.addNode(dataTreeNode);
	}

	/**
	 * Returns the file name to display.
	 * 
	 * @param source
	 * 
	 * @param defaultName
	 *            the default name.
	 * @return
	 */
	public static String makeDisplayName(Object source, String defaultName) {
		String name = WWIO.getSourcePath(source);
		if (name != null)
			name = WWIO.getFilename(name);
		if (name == null)
			name = defaultName;

		return name;
	}

	/**
	 * Loads file to the World Wind.
	 * 
	 * @param value
	 *            local file path or url.
	 * @param uuid
	 *            if this data is loaded from the JDrawPanle, this is the id of
	 *            JDrawPanel or is null.
	 * @return
	 */
	public static boolean addToWorldWind(String value, String uuid) {
		JTabbedPane tabbedPane = GlobeParam.getDisplayJTabbedPanel();
		tabbedPane.setSelectedIndex(0);
		String baseDir = PathConf.getDataDir();
		File addFile = null;
		String dir = null;
		
		DataLayerPanel dataLayerPanel = GlobeParam.getDataLayerPanel();
		DataTreeNode treeNode= dataLayerPanel.contains(value);
		if (treeNode!=null) {
			JTree tree = dataLayerPanel.getjTree();
			TreePath path = new TreePath(treeNode.getPath());
			tree.setSelectionPath(path);
			return true;
		}
		
		if (value.startsWith("http:") || value.startsWith("ftp:")) {
			String fileName = FileTool.getFileName(value);
			if (BaseTool.isNull(uuid)) {
				dir = GlobeParam.DEFAULT_UUID;
			} else {
				dir = uuid;
			}
			// File fileDownload = new File(baseDir + File.separator +
			// fileName);
			File dirFile = new File(baseDir + File.separator + dir);
			if (!dirFile.exists()) {
				dirFile.mkdirs();
			}
			File fileDownload = new File(baseDir + File.separator + dir
					+ File.separator + fileName);
			/*int i = 0;
			while (fileDownload.exists()) {
				fileDownload = new File(baseDir + File.separator + (i++) + "_"
						+ fileName);
			}*/
			if (!fileDownload.exists()) {
				DownloadMonitorDialog dialog = DownloadMonitorDialog.getInstance();
				dialog.setVisible(true);
				dialog.setMessage("Downloading file.");
				BaseTool.downZIP(fileDownload.getAbsolutePath(), value);
				dialog.setMessage("Downloaded successfully");
			}
			addFile = fileDownload;
		} else {
			addFile = new File(value);
		}
		
		
		DataLayerInfo dataNode = new DataLayerInfo();
		if (!BaseTool.isNull(uuid)) {
			dataNode.setUuid(uuid);
		}
		dataNode.setPath(value);
		IFileLoad fileLoad = FileLoadFactory.create(addFile);

		if (fileLoad == null) {
			logger.error("Failed to find a loader to load " + value);
			JOptionPane.showMessageDialog(null,
					"Failed to find a file loader for this file.");
			return false;
		}
		fileLoad.setDataNode(dataNode);
		fileLoad.setWWDPanel(GlobeParam.getWwPanel());
		fileLoad.execute();
		return true;
	}

	/**
	 * Deletes data layer from the WorldWindow.
	 * 
	 * @param layer
	 * @param wwd
	 */
	public static void deleteLayer(Layer layer, WorldWindow wwd) {
		LayerList layers = wwd.getModel().getLayers();
		if (layers.contains(layer)) {
			layers.remove(layer);
		}
	}

	/**
	 * Adds a sensor balloon to the WorldWind.
	 * 
	 * @param sensorBean
	 * @return
	 */
	public static boolean addBalloon(SOSSensor sensorBean) {
		JTabbedPane tabbedPane = GlobeParam.getDisplayJTabbedPanel();
		tabbedPane.setSelectedIndex(0);
		WWDPanel wwdPanel = GlobeParam.getWwPanel();
		SensorBalloon sensorBalloon = new SensorBalloon(wwdPanel);
		sensorBalloon.add(sensorBean);
		return true;
	}

	/**
	 * Views the provenance of the data node.
	 * 
	 * @param dataNode
	 */
	public static void viewProv(DataLayerInfo dataNode) {
		String uuid = dataNode.getUuid();
		String path = dataNode.getPath();
		if (BaseTool.isNull(uuid)) {
			JOptionPane.showMessageDialog(GlobeParam.getClient(), "Not used");
			return;
		}
		JTabbedPane tabbedPane = GlobeParam.getDisplayJTabbedPanel();
		for (int i = 0; i < tabbedPane.getTabCount(); i++) {
			Component com = tabbedPane.getComponentAt(i);
			if (com instanceof JScrollPane) {
				JScrollPane scrollPane = (JScrollPane) com;
				JViewport viewport = scrollPane.getViewport();
				JDrawPanel drawPanel = (JDrawPanel) viewport.getComponent(0);
				if (drawPanel.getUuid().equals(uuid)) {
					tabbedPane.setSelectedIndex(i);
					if (BaseTool.isNull(path)) {
						return;
					}
					PureModelStore modelStore = drawPanel.getModelStore();
					FigureStore figureStore = drawPanel.getFigureStore();
					List<AtomProcess> atomProcessList = modelStore
							.getAtomProcesses();
					OutPutParam outPutParam = null;
					String value;
					for (AtomProcess atom : atomProcessList) {
						outPutParam = atom.getOutPutParam();
						value = outPutParam.getValue();
						if (!BaseTool.isNull(value) && value.equals(path)) {
							ProcessFigure processFigure = figureStore
									.getProcessFigureByUuid(atom.getUUID());
							DataFigure dataFigure = processFigure
									.getDataFigureByUuid(atom.getUUID() + "#"
											+ outPutParam.getName());
							if (dataFigure != null) {
								FigureUtility figureUtility = FigureUtility
										.getInstance();
								dataFigure.setChoosen(true);
								figureUtility.clearOtherChoosen(dataFigure,
										figureStore.getAllFigures());
								drawPanel.repaint();
								return;
							}
						}
						List<ComplexParam> complexParams = atom
								.getComplexParam();
						for (ComplexParam complexParam : complexParams) {
							value = complexParam.getValue();
							if (!BaseTool.isNull(value) && value.equals(path)) {
								ProcessFigure processFigure = figureStore
										.getProcessFigureByUuid(atom.getUUID());
								DataFigure dataFigure = processFigure
										.getDataFigureByUuid(atom.getUUID()
												+ "#" + complexParam.getName());
								if (dataFigure != null) {
									FigureUtility figureUtility = FigureUtility
											.getInstance();
									dataFigure.setChoosen(true);
									figureUtility.clearOtherChoosen(dataFigure,
											figureStore.getAllFigures());
									drawPanel.repaint();
									return;
								}
							}
						}
					}
				}
			}

		}
	}
}
